@@ -91,35 +91,48 @@ sub set_output_entry {
   my $secnum = $section->number;
   my $key = $be->get_field('citekey');
 
-  # Make the right casing function
-  my $casing;
-
+  # Make the right casing/output mapping function
+  my $outmap;
   if (Biber::Config->getoption('output_fieldcase') eq 'upper') {
-    $casing = sub {uc(shift)};
+    $outmap = sub {my $f = shift; uc($CONFIG_OUTPUT_FIELDREPLACE{$f} // $f)};
   }
   elsif (Biber::Config->getoption('output_fieldcase') eq 'lower') {
-    $casing = sub {lc(shift)};
+    $outmap = sub {my $f = shift; lc($CONFIG_OUTPUT_FIELDREPLACE{$f} // $f)};
   }
   elsif (Biber::Config->getoption('output_fieldcase') eq 'title') {
-    $casing = sub {ucfirst(shift)};
+    $outmap = sub {my $f = shift; ucfirst($CONFIG_OUTPUT_FIELDREPLACE{$f} // $f)};
   }
 
   $acc .= '@';
-  $acc .= $casing->($bee);
+  $acc .= $outmap->($bee);
   $acc .=  "\{$key,\n";
 
   # hash accumulator so we can gather all the data before formatting so that things like
   # $max_field_len can be calculated
   my %acc;
 
+  # IDs
+  if (my $val = $be->get_field('ids')) {
+    $acc{$outmap->('ids')} = join(',', $val->@*);
+  }
+
   # Name fields
-  my $tonamesub = 'name_to_bib';
+  my $tonamesub = 'name_to_bibtex';
   if (Biber::Config->getoption('output_xname')) {
     $tonamesub = 'name_to_xname';
   }
 
   foreach my $namefield ($dmh->{namelists}->@*) {
     if (my $names = $be->get_field($namefield)) {
+
+      # XDATA is special
+      unless (Biber::Config->getoption('output_resolve_xdata')) { # already resolved
+        if (my $xdata = $names->get_xdata) {
+          $acc{$outmap->($namefield)} = xdatarefout($xdata);
+          next;
+        }
+      }
+
       my $namesep = Biber::Config->getoption('output_namesep');
       my @namelist;
 
@@ -135,13 +148,23 @@ sub set_output_entry {
 
       # Now add all names to accumulator
       foreach my $name ($names->names->@*) {
+
+        # XDATA is special
+        unless (Biber::Config->getoption('output_resolve_xdata')) {
+          if (my $xdata = $name->get_xdata) {
+            push @namelist, xdatarefout($xdata);
+            next;
+          }
+        }
+
         push @namelist, $name->$tonamesub;
       }
-      $acc{$casing->($namefield)} = join(" $namesep ", @namelist);
+
+      $acc{$outmap->($namefield)} = join(" $namesep ", @namelist);
 
       # Deal with morenames
       if ($names->get_morenames) {
-        $acc{$casing->($namefield)} .= " $namesep others";
+        $acc{$outmap->($namefield)} .= " $namesep others";
       }
     }
   }
@@ -152,9 +175,13 @@ sub set_output_entry {
       my $listsep = Biber::Config->getoption('output_listsep');
       my @plainlist;
       foreach my $item ($list->@*) {
+        unless (Biber::Config->getoption('output_resolve_xdata')) {
+          my $xd = xdatarefcheck($item);
+          $item = $xd // $item;
+        }
         push @plainlist, $item;
       }
-      $acc{$casing->($listfield)} = join(" $listsep ", @plainlist);
+      $acc{$outmap->($listfield)} = join(" $listsep ", @plainlist);
     }
   }
 
@@ -163,27 +190,31 @@ sub set_output_entry {
   foreach my $opt (Biber::Config->getblxentryoptions($secnum, $key)) {
     push @entryoptions, $opt . '=' . Biber::Config->getblxoption($secnum, $opt, undef, $key);
   }
-  $acc{$casing->('options')} = join(',', @entryoptions) if @entryoptions;
+  $acc{$outmap->('options')} = join(',', @entryoptions) if @entryoptions;
 
   # Date fields
   foreach my $d ($dmh->{datefields}->@*) {
     $d =~ s/date$//;
     next unless $be->get_field("${d}year");
-    $acc{$casing->("${d}date")} = construct_datetime($be, $d);
-  }
 
-  # YEAR and MONTH are legacy - convert these to DATE if possible
-  if (my $val = $be->get_field('year')) {
-    if (looks_like_number($val)) {
-      $acc{$casing->('date')} = $val;
-      $be->del_field('year');
-    }
-  }
-  if (my $val = $be->get_field('month')) {
-    if (looks_like_number($val)) {
-      $acc{$casing->('date')} .= "-$val";
-      $be->del_field('month');
+    # Output legacy dates for YEAR/MONTH if requested
+    if (not $d and Biber::Config->getoption('output_legacy_dates')) {
+      if (my $val = $be->get_field('year')) {
+        if (not $be->get_field('day') and
+            not $be->get_field('endyear')) {
+          $acc{$outmap->('year')} = $val;
+          if (my $mval = $be->get_field('month')) {
+            $acc{$outmap->('month')} = $mval;
+          }
+          next;
+        }
+        else {
+          biber_warn("Date in entry '$key' has DAY or ENDYEAR, cannot be output in legacy format.");
+        }
+      }
     }
+
+    $acc{$outmap->("${d}date")} = construct_datetime($be, $d);
   }
 
   # If CROSSREF and XDATA have been resolved, don't output them
@@ -201,7 +232,11 @@ sub set_output_entry {
   # Standard fields
   foreach my $field ($dmh->{fields}->@*) {
     if (my $val = $be->get_field($field)) {
-      $acc{$casing->($field)} = $val;
+      unless (Biber::Config->getoption('output_resolve_xdata')) {
+        my $xd = xdatarefcheck($val);
+        $val = $xd // $val;
+      }
+      $acc{$outmap->($field)} = $val;
     }
   }
 
@@ -211,34 +246,53 @@ sub set_output_entry {
     # output with its own special macro below
     next if $field eq 'keywords';
     if (my $f = $be->get_field($field)) {
-      $acc{$casing->($field)} .= join(',', $f->@*);
+      my $fl = join(',', $f->@*);
+      unless (Biber::Config->getoption('output_resolve_xdata')) {
+        my $xd = xdatarefcheck($fl);
+        $fl = $xd // $fl;
+      }
+      $acc{$outmap->($field)} .= $fl;
     }
   }
 
   # Ranges
   foreach my $rfield ($dmh->{ranges}->@*) {
     if ( my $rf = $be->get_field($rfield) ) {
-      $acc{$casing->($rfield)} .= construct_range($rf);
+      my $rfl = construct_range($rf);
+      unless (Biber::Config->getoption('output_resolve_xdata')) {
+        my $xd = xdatarefcheck($rfl);
+        $rfl = $xd // $rfl;
+      }
+      $acc{$outmap->($rfield)} .= $rfl;
     }
   }
 
   # Verbatim fields
   foreach my $vfield ($dmh->{vfields}->@*) {
     if ( my $vf = $be->get_field($vfield) ) {
-      $acc{$casing->($vfield)} = $vf;
+      unless (Biber::Config->getoption('output_resolve_xdata')) {
+        my $xd = xdatarefcheck($vf);
+        $vf = $xd // $vf;
+      }
+      $acc{$outmap->($vfield)} = $vf;
     }
   }
 
   # Keywords
   if ( my $k = $be->get_field('keywords') ) {
-    $acc{$casing->('keywords')} = join(',', $k->@*);
+    my $kl = join(',', $k->@*);
+    unless (Biber::Config->getoption('output_resolve_xdata')) {
+      my $xd = xdatarefcheck($kl);
+      $kl = $xd // $kl;
+    }
+    $acc{$outmap->('keywords')} = $kl;
   }
 
   # Annotations
   foreach my $f (keys %acc) {
     if (Biber::Annotation->is_annotated_field($key, lc($f))) {
       foreach my $n (Biber::Annotation->get_annotation_names($key, lc($f))) {
-        $acc{$casing->($f) . Biber::Config->getoption('output_annotation_marker') .
+        $acc{$outmap->($f) . Biber::Config->getoption('output_annotation_marker') .
             Biber::Config->getoption('output_named_annotation_marker') . $n} = construct_annotation($key, lc($f), $n);
       }
     }
@@ -268,8 +322,8 @@ sub set_output_entry {
       }
       delete @acc{@donefields};
     }
-    elsif (my $value = delete $acc{$casing->($field)}) {
-      $acc .= bibfield($casing->($field), $value, $max_field_len);
+    elsif (my $value = delete $acc{$outmap->($field)}) {
+      $acc .= bibfield($outmap->($field), $value, $max_field_len);
     }
   }
 
@@ -661,7 +715,6 @@ __END__
 
 =head1 AUTHORS
 
-François Charette, C<< <firmicus at ankabut.net> >>
 Philip Kime C<< <philip at kime.org.uk> >>
 
 =head1 BUGS
@@ -671,7 +724,8 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2019 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2012 François Charette and Philip Kime, all rights reserved.
+Copyright 2012-2020 Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
